Package: SemDist
Version: 1.45.0
Date: 2014-09-04
Title: Information Accretion-based Function Predictor Evaluation
Author: Ian Gonzalez and Wyatt Clark
Maintainer: Ian Gonzalez <gonzalez.isv@gmail.com>
Depends: R (>= 3.1), AnnotationDbi, GO.db, annotate
Suggests: GOSemSim
Description: This package implements methods to calculate information
  accretion for a given version of the gene ontology and uses this
  data to calculate remaining uncertainty, misinformation, and
  semantic similarity for given sets of predicted annotations and
  true annotations from a protein function predictor.
biocViews: Classification, Annotation, GO, Software
License: GPL (>= 2)
URL: http://github.com/iangonzalez/SemDist
git_url: https://git.bioconductor.org/packages/SemDist
git_branch: devel
git_last_commit: f13588e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:08:42 UTC; biocbuild
