#SemDist
=======

R bioconductor package for computing semantic distance statistic for protein function predictors.

##File list/functions:
--------------------
###computeIA.R : 
This file contains the computeIA function, which creates a file containing all of the information accretion data for the input (organism, ontology). Both inputs are strings.

###findRuMi.R : 	
This file contains the computeMI and computeRU functions, which calculate the remaining uncertainty and misinformation for given sets of true and predicted terms in a specified ontology.

###gene2GO.R :
Contains additional helper functions to load GO data.

###utilities.R :
Contains helper functions to initialize package environemnts.